// This file is part of Hangfire.
// Copyright  2013-2014 Sergey Odinokov.
// 
// Hangfire is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as 
// published by the Free Software Foundation, either version 3 
// of the License, or any later version.
// 
// Hangfire is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public 
// License along with Hangfire. If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;
using System.Threading;
using Hangfire.Annotations;
using Hangfire.Storage;

namespace Hangfire.States
{
    public class StateChangeContext
    {
        public StateChangeContext(
            [NotNull] JobStorage storage,
            [NotNull] IStorageConnection connection,
            [NotNull] string backgroundJobId, 
            [NotNull] IState newState)
            : this(storage, connection, backgroundJobId, newState, null)
        {
        }

        public StateChangeContext(
            [NotNull] JobStorage storage,
            [NotNull] IStorageConnection connection,
            [NotNull] string backgroundJobId, 
            [NotNull] IState newState, 
            [CanBeNull] params string[] expectedStates)
            : this(storage, connection, backgroundJobId, newState, expectedStates, CancellationToken.None)
        {
        }

        public StateChangeContext(
            [NotNull] JobStorage storage, 
            [NotNull] IStorageConnection connection,
            [NotNull] string backgroundJobId, 
            [NotNull] IState newState, 
            [CanBeNull] IEnumerable<string> expectedStates, 
            CancellationToken cancellationToken)
        {
            if (storage == null) throw new ArgumentNullException("storage");
            if (connection == null) throw new ArgumentNullException("connection");
            if (backgroundJobId == null) throw new ArgumentNullException("backgroundJobId");
            if (newState == null) throw new ArgumentNullException("newState");

            Storage = storage;
            Connection = connection;
            BackgroundJobId = backgroundJobId;
            NewState = newState;
            ExpectedStates = expectedStates;
            CancellationToken = cancellationToken;
        }

        public JobStorage Storage { get; private set; }
        public IStorageConnection Connection { get; private set; }
        public string BackgroundJobId { get; private set; }
        public IState NewState { get; private set; }
        public IEnumerable<string> ExpectedStates { get; private set; }
        public CancellationToken CancellationToken { get; private set; }
    }
}